#ev-fullscreen-toolbar {

}

#ev-loading-message {
    background-color: @theme_selected_bg_color;
    color: @theme_selected_fg_color;
    border-radius: 3px;
    padding: 8px;
}

evview.document-page {
    background-color: white;
    border-color: black;
    border-style: solid;
    border-width: 1px;
    border-radius: 0px;
    border-image: none;
    padding: 0;
}

evview.document-page:active {
    background-color: white;
    border-color: @theme_selected_bg_color;
    border-style: solid;
    border-width: 1px;
    border-radius: 0px;
    border-image: none;
    padding: 0;
}

evview.document-page.inverted {
    background-color: black;
}

evpresentationview {
    background-color: black;
}

evpresentationview.white-mode {
    background-color: white;
}

